
# Simplified methods
from .fire_base import df_conc
from .fire_base import df_steel
from .fire_base import sig_kc
from .fire_base import sig_ks
from .fire_base import stemp

# Tabulated
# Functions for tabulated methods
from .fire_base import delta_a

# Tables for tabulated methods
from .fire_base import wall_a40rei
from .fire_base import wall_h40rei
from .fire_base import wall_h40r
from .fire_base import wall_a40r
from .fire_base import wall_h25rei
from .fire_base import wall_h25r
from .fire_base import wall_a25rei
from .fire_base import wall_a25r
from .fire_base import beam_bsimp
from .fire_base import beam_bwsimp
from .fire_base import beam_asimp



import math
import matplotlib.pyplot as plt
import matplotlib as mpl
from matplotlib.ticker import (AutoMinorLocator, MultipleLocator)
import matplotlib.image as mpimg
from matplotlib.offsetbox import (TextArea, DrawingArea, OffsetImage,AnnotationBbox)
import matplotlib.path as mpltPath
from matplotlib import rc
import numpy as np
from itertools import product

# font.family        : serif
# font.serif         : Times, Palatino, New Century Schoolbook, Bookman, Computer Modern Roman
# font.sans-serif    : Helvetica, Avant Garde, Computer Modern Sans serif
# font.cursive       : Zapf Chancery
# font.monospace     : Courier, Computer Modern Typewriter
# text.usetex        : true
rc('font',**{'family':'serif','serif':['Computer Modern Roman']})
## for Palatino and other serif fonts use:
#rc('font',**{'family':'serif','serif':['Palatino']})
rc('text', usetex=True)

fck = 50
fyk = 500
rec = 0.05
concr = "C50/60"
steel = "A500"
Es = 200000

num_bars = 6
fcd = 1
radius = 0.5
diameter = 2.0 * radius
num_layers = 100
height_layer = diameter / num_layers
radius_reinf = radius - rec
    
areat = math.pi * radius * radius
fyd = round (fyk/1.15, 0)
eta = 1 if fck <= 50 else 1.0 + (fck - 50) / 200
lambd = 0.8 if fck <= 50 else 0.8 + (fck - 50) / 400
epsc2 = 2 if fck <= 50 else 2.0 + 0.085 * math.pow(fck - 50.0, 0.53)
epscu2 = 3.5 if fck <= 50 else 2.5 + 35 * math.pow((90.0 - fck)/100.0, 4)
n = 2 if fck <= 50 else 1.4 + 23.4 * math.pow((90.0 - fck)/100.0, 4)
x2 = 1.0 - epsc2/epscu2


concrete = {
    "C50/60": 50,
    "C55/67": 55,
    "C60/75": 60,
    "C70/85": 70,
    # "C80/95": 80,
}

reinfsteel = {
    "A400": 400,
    "A500": 500
}

steelname = {
    "A400": ["A400","S400"],
    "A500": ["A500","S500"]
}

locale = {
    "PT": 0,
    "EN": 1
}

zetas = [0.05, 0.1, 0.15]

nns = [100, 6, 10]

limits = {   #  w   miu  niu  niu
    "C50/60": [1.0, 0.4, 2.1, -1.1],
    "C55/67": [0.5, 0.25, 1.6, -0.6],
    "C60/75": [0.5, 0.25, 1.6, -0.6],
    "C70/85": [0.5, 0.2, 1.6, -0.6]
}

def sign(num):
    return 1 if num >= 0 else 0 if num == 0 else -1

def epsilon(x, x0):
    return epscu2 * (x0-x) / x0 if x0 < 1 else epsc2 * (1.0 - (x - x2)/(x0 - x2))

def conc_stress(epsc):
    return (1-((1-epsc/epsc2)**n))


def concreteforces(x0):
    ndiv = 1000
    ddiv = 2.0*radius / ndiv
    xxi = ddiv / 2.0
    
    xxx = np.linspace(ddiv / 2.0, x0, ndiv)
    yyy = np.subtract(radius, xxx)

    mtot = 0
    ftot = 0
    for i in range(ndiv):
        xx  = xxi + i *  ddiv
        if xx > x0: break
        eps = epsilon(xx, x0)
        dist = radius - xx

        area = ddiv * 2.0*math.sqrt(radius*radius-dist*dist)
        sig = fcd if eps >= epsc2 else (1-((1-eps/epsc2)**n))

        ftot += area * sig
        mtot += area * sig * dist

    return (ftot / areat, mtot / areat)


def steelforces(x0, ns=6):
    epsyd = fyd / Es * 1000

    dtheta = 2.0 * np.pi / ns
    theta0 = 0.5 * (1-ns%2) * dtheta
    r = radius - rec    
    polygon = radius-r*np.array([[np.cos(x)] for x in np.linspace(theta0,theta0+2*np.pi,ns+1)[:-1]])   
    eps = epsilon(polygon, x0)

    ftot = 0
    mtot = 0
    for i in range(ns):
        theta = theta0 + i * dtheta
        r = radius - rec
        rs = r * math.cos(theta)
        z = radius - rs
        eps = epsilon(z, x0)
        fs = sign(eps)*fyd if math.fabs(eps) >= epsyd else eps * Es / 1000
        ftot += fs
        mtot += fs * rs

    return (round(ftot / ns / fyd,6), round(mtot / ns / fyd,6))


def bend_circular():
    # calculate secrion coordinates
    xxi = height_layer / 2.0
    conc_dist = np.linspace(xxi, diameter-xxi, num_layers)
    conc_aux = np.subtract(radius, conc_dist)
    conc_area = height_layer * 2.0 * np.sqrt(np.subtract(radius*radius, conc_aux*conc_aux))
    conc_mom = conc_area * conc_dist
    
    # calculate reinforcement coordinates
    dtheta = 2.0 * np.pi / num_bars
    theta0 = 0.5 * (1-num_bars%2) * dtheta 
    reinf_dist = radius-radius_reinf*np.array([[np.cos(x)] for x in np.linspace(theta0,theta0+2*np.pi,num_bars+1)[:-1]])

    return conc_dist, conc_area, conc_mom, reinf_dist



if __name__ == "__main__":
    conc_dist, conc_area, conc_mom, reinf_dist = bend_circular()
    
    num_axis_pos = 100
    # neutralaxispositions
    x0 = np.concatenate((np.linspace(0.0001, 1, num_axis_pos), np.geomspace(1.0001, 100000, num_axis_pos)))
    # strains in eaxch concrete layer
    epsc = np.array([epsilon(conc_dist, xx) for xx in x0])
    epsc[epsc < 0] = 0.0
    # steress in eaxch concrete layer
    ### numpy compare and replace by number

    sig = np.where(epsc > epsc2, fcd, conc_stress(epsc))

    # inputs

    #ns = 6
    props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)
    props2 = dict(boxstyle='round', pad=0, ec="white", facecolor='white', alpha=0.8)
    props3 = dict(boxstyle='round', pad=0, ec="white", facecolor='white', alpha=0.8)

    listofinputs = [concrete, reinfsteel, zetas, nns]

    dw = 0.02
    numberw = int(limits[concr][0] / dw)
    aspectratio = (9.0/7.5)/((limits[concr][2]-limits[concr][3])/limits[concr][1])

    print('{0} {1} {2:.3f}'.format(fck, fyk,rec))

    # outputs
    # ciclo aos w (constroi linhas)
    wlist2 = []
    alist = []
    #ax.set_aspect(1.5)
    plt.figure(figsize = (7.5,9))
    ax = plt.gca()

    axlist = []
    aylist = []
    for i in range(numberw+1):
        w = i * dw
        mlist2 = []
        nlist2 = []
        # ciclo aox x (constroi uma linha)
        for j in range(num_axis_pos):
            x0 = 2.0 * j / num_axis_pos + 0.00001 if x0 <= 1 else 1 / (2.0 - x0)

            conc2 = concreteforces(x0)
            reinf = steelforces(x0, num_bars)
            mlist2.append(conc2[1]+reinf[1]*w)
            nlist2.append(conc2[0]+reinf[0]*w)

        wlist2.append([mlist2,nlist2])

        #print("w = " + str(w))
        s = 1
        # plt.plot(wlist[i][0], wlist[i][1], s, c="C"+str(i+1), alpha=0.5)
        # plt.plot(wlist[i][0], wlist[i][1], s, c='b', alpha=0.5)
        linew = 1.5 if i%5 == 0 else 1
        plt.plot(wlist2[i][0], wlist2[i][1], s, c='r', alpha=0.5, linewidth=linew)

        if i%5 == 0:
            ax.text(0.003, wlist2[i][1][0], '{:.1f}'.format(w),
                horizontalalignment='left', verticalalignment='bottom', bbox=props2)

    for j in range(16):
        #if j == 0: continue
        axlist = []
        aylist = []
        # ciclo aox x (constroi uma linha)
        for i in range(2):
            w = 0 if i == 0 else limits[concr][0] + dw
            x0 = 2.0 * j / 20 + 0.00001
            if x0 > 1.0: 
                x0 = 1 / (2.0 - x0)

            conc = concreteforces(x0)
            reinf = steelforces(x0, num_bars)
            axlist.append(conc[1]+reinf[1]*w)
            aylist.append(conc[0]+reinf[0]*w)

            if i == 1 and j > 0:
                lim = ax.get_xlim()
                dx = 7.5*(axlist[1]-axlist[0]) / (lim[1]-lim[0])
                dy = 9.0*(aylist[1]-aylist[0]) / (limits[concr][2]-limits[concr][3])
                angle = math.atan2(dy, dx) * 180.0 / math.pi
                aa = round(x0, 2)
                pos = 'top' if x0 < 0.6 else 'bottom'
                ax.text(axlist[1]+0.002, aylist[1]+0.002, r'$\alpha$={0:.2f}'.format(aa),
                    horizontalalignment='left', verticalalignment=pos, rotation = angle, bbox=props3)

        alist.append([axlist,aylist])

        #print("a = " + str(x0))
        s = 1
        # plt.plot(wlist[i][0], wlist[i][1], s, c="C"+str(i+1), alpha=0.5)
        plt.plot(alist[j][0], alist[j][1], s, c='b', alpha=0.5)

    axes = plt.gca()
    figure = plt.gcf()
    loc = axes.get_position()
    nstr = str(num_bars) if num_bars < 20 else r'$\infty$'
    strtext = ''.join(('{0}\n{1}\n'.format(concr, steel),r' $\zeta$ = {0:.2f}'.format(rec),'\n',r'$n_s$ = {}'.format(nstr)))
    # figure.text(0.8, 0.9, "C20/25", fontsize=12, bbox=props)
    # figure.text(0.5, 0.98, 'Flexão composta em secções retangulares', fontsize=14, horizontalalignment='center', verticalalignment='top')
    # figure.text(0.5, 0.95, 'de acordo com a EN 1992-1-1:2004', fontsize=10, horizontalalignment='center', verticalalignment='top')
    figure.text(0.9, 0.97, strtext, fontsize=10, horizontalalignment='center', verticalalignment='top', bbox=props)

    sleft = 0.3
    stop = 0.96
    strtext = r'$\nu=\frac{N_{Rd}}{A_cf_{cd}}$'
    figure.text(sleft, stop, strtext, fontsize=12, horizontalalignment='left', verticalalignment='center')
    strtext = r'$\mu=\frac{M_{Rd}}{A_cDf_{cd}}$'
    figure.text(sleft, stop - 0.05, strtext, fontsize=12, horizontalalignment='left', verticalalignment='center')
    strtext = r'$\omega=\frac{A_{s}f_{yd}}{A_cf_{cd}}$'
    figure.text(sleft+0.13, stop, strtext, fontsize=12, horizontalalignment='left', verticalalignment='center')
    strtext = r'$\zeta=\frac{c}{D}$'
    figure.text(sleft+0.13, stop - 0.05, strtext, fontsize=12, horizontalalignment='left', verticalalignment='center')
    strtext = r'$\alpha=\frac{x}{D}$'
    figure.text(sleft+0.26, stop - 0.05, strtext, fontsize=12, horizontalalignment='left', verticalalignment='center')

    plt.xlabel("Momento fletor, " + r'$\mu$')
    plt.ylabel("Força axial, " + r'$\nu$')
    #plt.xlim(0,limits[concr][1])
    plt.ylim(limits[concr][3],limits[concr][2])
    # plt.legend(legend, loc='upper right')
    plt.minorticks_on()
    plt.grid(b=True, color='black', which='minor', alpha=0.15)
    plt.grid(b=True, color='black', which='major')

    if num_bars == 6:
        im = plt.imread('figs/flexaocomposta-c6.png')
    elif num_bars == 10:
        im = plt.imread('figs/flexaocomposta-c10.png')
    else:
        im = plt.imread('figs/flexaocomposta-c.png')
    newax = figure.add_axes([0.05, 0.88, 0.25, 0.12], anchor='SW', zorder=-1)
    newax.imshow(im)
    newax.axis('off')

    # plt.xticks(np.arange(0,0.4,0.02))
    # plt.yticks(np.arange(-1,1,0.02))
    filename = 'circcol-pt/circ_col_' + str(fck) + '_' + str(fyk) + '_' + '{0:.0f}_{1:.0f}'.format(rec*100,num_bars) + '.svg'
    plt.savefig(fname=filename, dpi=600)
    #plt.show()

    print("End")



import json

# Turns a dictionary into a class
# declaring a class
class obj:

    # constructor
    def __init__(self, dict1):
        self.__dict__.update(dict1)


def dict2obj(dict1):
    # using json.loads method and passing json.dumps
    # method and custom object hook as arguments
    return json.loads(json.dumps(dict1), object_hook=obj)




import math
import numpy as np


def calc_reinf_plane(n_xx: float, n_yy:float, n_xy: float)->list:
    """Calculate the reinforcement in a plane element.

    Args:
        n_xx (float): axial force in x direction
        n_yy (float): axial force in y direction
        n_xy (float): shear force in xy direction

    Returns:
        list: the reinforecment in both diretions and concrete stresses
    """
    abs_n_xy = abs(n_xy)
    n_xx_n_yy = n_xx * n_yy
    n_xy_2 = n_xy * n_xy
    if n_xx >= -abs_n_xy and n_yy >= -abs_n_xy:
        theta = 1.0
        asx = n_xx + abs_n_xy
        asy = n_yy + abs_n_xy
        asc = 2.0 * abs_n_xy
    elif n_xx < -abs_n_xy and n_xx <= n_yy and n_xx_n_yy <= n_xy_2:
        theta = 0 if abs_n_xy < 1.0e-12 else -n_xx / abs_n_xy
        asx = 0.0
        asy = n_yy + n_xy_2 / abs(n_xx)
        asc = abs(n_xx) * (1.0 + (abs_n_xy/n_xx)**2)
    elif n_yy < -abs_n_xy and n_xx >= n_yy and n_xx_n_yy <= n_xy_2:
        theta = -abs_n_xy / n_yy
        asx = n_xx + n_xy_2 / abs(n_yy)
        asy = 0.0
        asc = abs(n_yy) * (1.0 + (abs_n_xy/n_yy)**2)
    else:
        cen = (n_xx + n_yy) * 0.5
        rad = math.sqrt(n_xy_2+0.25*(n_xx-n_yy)**2)
        theta = math.atan2(n_xx-n_yy, 2*n_xy) / 2.0
        theta = 0 if theta == 0 else 1.0/math.tan(theta)
        asx = 0.0
        asy = 0.0
        asc = abs(cen - rad)
        
    return [asx, asy, asc, theta]


def cal_reinf_shell_plan(n_t_xx: float, n_t_yy: float, n_t_xy: float, 
                n_b_xx: float, n_b_yy: float, n_b_xy: float)->np.ndarray:

    return np.array(calc_reinf_plane(n_t_xx, n_t_yy, n_t_xy) + calc_reinf_plane(n_b_xx, n_b_yy, n_b_xy))


def calc_reinf_shell(n_xx: float, n_yy: float, n_xy: float, m_xx: float, m_yy: float, m_xy: float,
            rec: float, h: float) -> np.ndarray:
    """Calculate the forces to ccalculate the reinforcement in a shell element.

    Args:
        n_xx (float): axial force in x direction
        n_yy (float): axial force in y direction
        n_xy (float): shear force in xy direction
        m_xx (float): moment in x direction (bending)
        m_yy (float): moment in y direction (bending)
        m_xy (float): moment in xy direction (torsion)
        rec (float): cover to reinforcement
        h (float): height of the shell

    Returns:
        np.array: the reinforecment in both diretions in top and bottom layer and concrete stresses
    """

    t = 2 * rec
    z = h - t
    if h - 4 * rec < 0: return np.array([math.nan,math.nan,math.nan,math.nan,math.nan,math.nan])
    
    n_t_xx = (0.5*n_xx + m_xx/z) / t
    n_t_yy = (0.5*n_yy + m_yy/z) / t
    n_t_xy = (0.5*n_xy - m_xy/z) / t

    n_b_xx = (0.5*n_xx - m_xx/z) / t
    n_b_yy = (0.5*n_yy - m_yy/z) / t
    n_b_xy = (0.5*n_xy + m_xy/z) / t

    as_vect = np.vectorize(cal_reinf_shell_plan, otypes=[np.ndarray])
    as_total = as_vect(n_t_xx, n_t_yy, n_t_xy, n_b_xx, n_b_yy, n_b_xy)
    return as_total



import numpy as np
import math


def principals(sigxx: float, sigyy: float, sigzz: float, 
               sigxy: float, sigyz: float, sigzx: float) -> np.ndarray:
     """Calculate the principal stresses and the normalized principal directions
     Author. Paulo Cachim (2022)

     Args:
          sigxx (float): stress xx
          sigyy (float): stress yy
          sigzz (float): stress zz
          sigxy (float): stress xy
          sigyz (float): stress yz
          sigzx (float): stress zx

     Returns:
          ndarray, ndarray: the principal stresses and the normalized principal directions
     """
     return np.linalg.eigh(np.array([[sigxx,sigxy,sigzx],
                         [sigxy,sigyy,sigyz],
                         [sigzx,sigyz,sigzz]]))


def principal_vectors(sigxx: float, sigyy: float, sigzz: float, 
               sigxy: float, sigyz: float, sigzx: float) -> np.ndarray:
     """Calculates the princcipal vectors (size proportional to principal stresses)
     Author. Paulo Cachim (2022)

     Args:
          sigxx (float): stress xx
          sigyy (float): stress yy
          sigzz (float): stress zz
          sigxy (float): stress xy
          sigyz (float): stress yz
          sigzx (float): stress zx

     Returns:
          ndarray: an array of the principal vectors
     """

     values, vectors = np.linalg.eigh(np.array([[sigxx,sigxy,sigzx],
                         [sigxy,sigyy,sigyz],
                         [sigzx,sigyz,sigzz]]))

     v = vectors * np.array([values[0]*vectors[0], 
                    values[1]*vectors[1], 
                    values[2]*vectors[2]])
     return v


def invariants(sigxx: float, sigyy: float, sigzz: float, 
                         sigxy: float, sigyz: float, sigzx: float) -> list:
     """Calculates the stress invariants
     Author. Paulo Cachim (2022)

     Args:
          sigxx (float): stress xx
          sigyy (float): stress yy
          sigzz (float): stress zz
          sigxy (float): stress xy
          sigyz (float): stress yz
          sigzx (float): stress zx

     Returns:
          stress invariants (list): I1, J2, J3, mean_stress, eqv_stress, 
                         lode_r, lode_z, lode_theta, cos3t, triaxiality
     """
     #stresses = [0.0,0.0,0.0,0.0,0.0,0.0]

     # load the stresses into our matrix and compute the 
     # deviatoric and isotropic stress matricies
     sigma = np.array([[sigxx,sigxy,sigzx],
                         [sigxy,sigyy,sigyz],
                         [sigzx,sigyz,sigzz]])
     sigma_iso = (np.trace(sigma)*np.eye(3))/3.0
     sigma_dev = sigma - sigma_iso

     # # compute max shear stress
     # maxshear = (max(eigvals)-min(eigvals))/2.0

     # compute the stress invariants
     I1 = np.trace(sigma)
     J2 = 0.5*np.trace(np.dot(sigma_dev,sigma_dev))
     J3 = 0.333333333333333333*np.trace(np.dot(sigma_dev,np.dot(sigma_dev,sigma_dev)))

     # compute other common stress measures
     mean_stress = I1/3.0
     eqv_stress  = math.sqrt(3.0*J2)

     # compute lode coordinates
     lode_r = math.sqrt(2.0*J2)
     lode_z = I1/math.sqrt(3.0)

     stresses = 3.0*math.sqrt(6.0)*np.linalg.det(sigma_dev/lode_r)
     if stresses < -1: stresses = -1
     if stresses > 1: stresses = 1
     lode_theta = 1.0/3.0*math.asin(stresses)
     cos3t = 2.5980762114*J3/(J2**1.5)

     # compute the stress triaxiality
     triaxiality = mean_stress/eqv_stress
     
     return I1, J2, J3, mean_stress, eqv_stress, lode_r, lode_z, lode_theta, cos3t, triaxiality


if __name__ == "__main__":

     print("\nTest stress module: principals:")
     eval, evec = principals(3.0, 2.0, -1.0, 0.0, 0.0, 0.0)
     print(eval)
     print(evec)
     print("\nTest stress module: principal_vectors:")
     evec = principal_vectors(3.0, 2.0, -1.0, 0.0, 0.0, 0.0)
     print(evec)

     print("\nTest stress module: stress invariants:")
     u = invariants(3.0, 2.0, -1.0, 0.3, -0.4, 0.5)
     print (u)


import json
import os
import pandas as pd
import importlib.resources as pkg_resources

"""This module contains the database of the Eurocodes

For convenience, the database is loaded as a dictionary and as a class. It allows to access the database in two ways:
1. Using the dictionary db (e.g. db["Materials"]["Timber"]["Classes"]["C24"]["fcd"])
2. Using the class dbase (e.g. dbase.Materials.Timber.Classes.C24.fcd)

There are also some functions to access the database:
- get_eurocodes(): returns the whole database
- get_materials(): returns the materials database
- get_timber(): returns the timber database
- get_timber_classes(): returns the timber classes database
- get_concrete(): returns the concrete database
- get_prestress(): returns the prestress database
- get_reinforcement(): returns the reinforcement database
- get_steel(): returns the steel database

And some variables:
- Eurocodes: the eurocodes database
- Concrete: the concrete database
- ConcreteClasses: the concrete classes database
- ConcreteParams: the concrete parameters database
- Reinforcement: the reinforcement database
- ReinforcementClasses: the reinforcement classes database
- ReinforcementBars: the reinforcement bars database
- ReinforcementParams: the reinforcement parameters database
- Steel: the steel database
- SteelParams: the steel parameters database
- SteelProfiles: the steel profiles database
- Bolts: the bolts database
- BoltClasses: the bolts classes database
- BoltDiameters: the bolts diameters database
- Prestress: the prestress database
- PrestressClasses: the prestress classes database
- PrestressParams: the prestress parameters database
- Timber: the timber database
- TimberClasses: the timber classes database
- TimberParams: the timber parameters database
- Materials: the materials database

"""

# Turns a dictionary into a class
# declaringa a class
class obj:
    # constructor
    def __init__(self, dict1):
        self.__dict__.update(dict1)


def dict2obj(dict1):
    # using json.loads method and passing json.dumps
    # method and custom object hook as arguments
    return json.loads(json.dumps(dict1), object_hook=obj)


database = {}
dirname = os.path.dirname(__file__)
db = json.loads(open(os.path.join(dirname,'eurocodes.json'),'r').read())["Eurocodes"]
db["SteelProfiles"]["Euro"] = json.loads(open(os.path.join(dirname,'prof_euro.json'),'r').read())

dbase = dict2obj(db)

Materials = db["Materials"]

Reinforcement = db["Materials"]["Reinforcement"]
ReinforcementClasses = Reinforcement["Classes"]
ReinforcementBars = Reinforcement["Rebars"]
ReinforcementParams = Reinforcement["Parameters"]
#ReinforcementClasses = db["Materials"]["Reinforcement"]["Classes"]

Concrete = db["Materials"]["Concrete"]
ConcreteClasses = Concrete["Classes"]
ConcreteParams = Concrete["Parameters"]

Prestress = db["Materials"]["Prestress"]
PrestressClasses = Prestress["Classes"]
PrestressParams = Prestress["Parameters"]

Timber = db["Materials"]["Timber"]
TimberClasses = Timber["Classes"]
TimberParams = Timber["Parameters"]
TimberLoadDuration = Timber["LoadDuration"]

Steel = db["Materials"]["Steel"]
SteelClasses = Steel["Classes"]
SteelParams = Steel["Parameters"]

Bolts = db["Materials"]["Bolts"]
BoltClasses = Bolts["Classes"]
BoltDiameters = Bolts["Diameters"]

SteelProfiles = db["SteelProfiles"]["Euro"]




from math import log10, log, exp


k_1 = 1.0 # coeficiente de turbulência

z0 = {"EU": {"0": 0.003, "I": 0.01, "II": 0.05, "III": 0.3, "IV": 1},
      "PT": {"I": 0.005, "II": 0.05, "III": 0.3, "IV": 1}}
zmin = {"EU": {"0": 1, "I": 1, "II": 2, "III": 5, "IV": 10},
        "PT": {"I": 1, "II": 3, "III": 8, "IV": 15}}

zz0 = z0["EU"]
zzmin = zmin["EU"]


def s_coef(x: float, z: float, H: float, Lu: float, Ld: float=1000) -> float:
    """_summary_

    Args:
        x (float): horizontal distance
        z (float): vertical distance
        H (float): height of the hill/cliff
        Lu (float): length of the hill/cliff
        Ld (float, optional): length of the cliff. Defaults to 1000, meaning it's a cliff.
    Returns:
        float: _description_
    """
    phi0 = H/Ld
    phi = H/Lu
    
    if phi < 0.05: return 0.0
    orography_type = 'hill' if phi0 > 0.05 else 'cliff'

    Le = H/0.3 if phi >= 0.3 else Lu
    x_Lu = x/Lu
    z_Le = z/Le

    s = 0.0
    if x_Lu <= 0:
        if -1.5 < x_Lu and z_Le < 2:
            A = 0.1552*z_Le**4 -0.8575*z_Le**3 +1.8133*z_Le**2 -1.9115*z_Le +1.0124
            B = 0.3542*z_Le**2 -1.0577*z_Le+2.6456
            s = A*exp(B*x_Lu)
        elif x_Lu < -1.5 or z_Le >= 2:
            s = 0.0
    else:
        if orography_type == 'hill':
            x_Ld = x/Ld
            if x_Ld < 2 and z_Le < 2:
                A = 0.1552*z_Le**4 -0.8575*z_Le**3 +1.8133*z_Le**2 -1.9115*z_Le +1.0124
                B = -0.3056*z_Le**2 +1.0212*z_Le -1.7637
                s = A*exp(B*x_Lu)
            elif x_Ld >= 2 or z_Le >= 2:
                s = 0.0
        else: # orography_type == 'cliff'
            x_Le = x/Le
            if z_Le < 0.1: z_Le = 0.1
            if (0.1 < x_Le < 3.5) and (z_Le < 2):
                logzle = log10(z_Le)
                A = -1.3420*logzle**3 -0.8222*logzle**2 +0.4609*logzle -0.0791
                B = -1.0196*logzle**3 -0.8910*logzle**2 +0.5343*logzle -0.1156
                C =  0.8030*logzle**3 +0.4236*logzle**2 -0.5738*logzle +0.1606
                logxle = log10(x/Le)
                s = A*logxle**2 + B*logxle + C
            elif (0.1 >= x_Le) and (z_Le < 2):
                s1 = 0.1552*z_Le**4 -0.8575*z_Le**3 +1.8133*z_Le**2 -1.9115*z_Le +1.0124
                logzle = log10(z_Le)
                A = -1.3420*logzle**3 -0.8222*logzle**2 +0.4609*logzle -0.0791
                B = -1.0196*logzle**3 -0.8910*logzle**2 +0.5343*logzle -0.1156
                C =  0.8030*logzle**3 +0.4236*logzle**2 -0.5738*logzle +0.1606
                logxle = log10(0.1/Le)
                s2 = A*logxle**2 + B*logxle + C
                s = s1 + x_Le*(s2-s1)/0.1           
            elif x_Le >= 3.5 or z_Le >= 2:
                s = 0.0
    return s


def c_0(z: float, x: float=0, H: float=0, Lu: float=10, Ld: float=1000) -> float:
    """Calculates the orography factor, taken as 1,0
    Args:
        z (float): vertical distance
        x (float, optional): horizontal distance. Defaults to 0.
        H (float, optional): height of the hill/cliff. Defaults to 0.
        Lu (float, optional): length of the hill/cliff. Defaults to 10.
        Ld (float, optional): length of the cliff. Defaults to 1000, meaning it's a cliff.

    Returns:
        float: orography factor
    """
    phi = H/Lu
    if phi < 0.05: return 1.0
    s = s_coef(x, z, H, Lu, Ld)
    return 1.0+2.0*s*phi if phi < 0.3 else 1.0+0.6*s


def c_r(z: float, z_min: float=1.0, z_0: float= 0.01) -> float:
    """ Calculate the roughness factor

    Args:
        z (float): vertical distance
        z_min (float, optional): minimum height. Defaults to 1.0.
        z_0 (float, optional): roughness length. Defaults to 0.01.

    Returns:
        float: the roughness factor
    """
    k_r = 0.19*((z_0/zz0["II"])**0.07)
    zeff = z if z >= z_min else z_min
    return k_r * log(zeff/z_0)


def v_b(vb_0: float, c_season: float=1.0, c_dir: float=1.0) -> float:
    """Calculates the basic wind velocity 

    Args:
        vb_0 (float): fundamental value of the basic wind velocity
        c_season (float, optional): seasonal factor. Defaults to 1.0.
        c_dir (float, optional): directional factor. Defaults to 1.0.

    Returns:
        float: basic wind velocity 
    """
    return c_season * c_dir * vb_0


def v_m(z: float, vb: float, zone: str) -> float:
    """ Calculates the mean wind velocity, vm(z), at a height z above the terrain.
    Depends on the terrain roughness and orography and on the basic wind velocity.

    Args:
        z (float): vertical distance
        vb (float): basic wind velocity
        zone (str): the terrain category

    Returns:
        float: mean wind velocity, vm(z) 
    """
    return c_r(z, zone) * c_0(z) * vb


def I_v(z: float, z_min: float=1.0, z_0: float= 0.01) -> float:
    """Calculates the turbulence intensity, Iv(z), at height z.
    It is defined as the standard deviation of the turbulence divided by the mean wind velocity.

    Args:
        z (float): vertical distance
        z_min (float, optional): minimum height. Defaults to 1.0.
        z_0 (float, optional): roughness length. Defaults to 0.01.

    Returns:
        float: turbulence intensity
    """
    zeff = z if z >= z_min else z_min
    Iv = k_1 / c_0(z) / log(zeff/z_0)
    return Iv


def q_p(z: float, vb0: float, z_min: float=1.0, z_0: float= 0.01, rho: float=1.25) -> float:
    """Calcculates the peak velocity pressure, qp(z), at height z, 
    which includes mean and short-term velocity fluctuations.

    Args:
        z (float): vertical distance
        vb0 (float): fundamental value of the basic wind velocity
        z_min (float, optional): minimum height. Defaults to 1.0.
        z_0 (float, optional): roughness length. Defaults to 0.01.
        rho (float, optional): air density. Defaults to 1.25 kg/m3.

    Returns:
        float: peak velocity pressure
    """
    # v = v_m(z, v_b(vb0), zone)
    v = c_r(z, z_min, z_0) * c_0(z) * vb0
    zeff = z if z >= z_min else z_min
    # Iv = I_v(z, zone)
    Iv = k_1 / c_0(z) / log(zeff/z_0)
    qp = 0.5 * (1.0 + 7*Iv) * v**2 * rho
    return qp



import numpy as np
import pandas as pd
import matplotlib.pyplot as plt


def get_spectrum_parameters(code: str, coef_imp: str, soil: str, zone: str) -> list:
    """Get the spectrum parameters

    Args:
        code (str): code to be used (CEN-1, CEN-2, PT-1, PT-2, PT-A)
                CEN-1, CEN-2: standard Eurocode spectrums
                PT-1, PT-2, PT-A: Portuguese National Annex spectrums
                    (PT-1 and PT-2, for continent and Madeira, PT-A for Azores)
        coef_imp (str): importance coefficient (i, ii, iii, iv)
        soil (str): soil type (A, B, C, D, E)
        zone (str): zone (1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 2.1, 2.2, 2.3, 2.4, 2.5, .1g, .2g, .3g, .4g, .5g, .6g, .7g, .8g, .9g, 1.0g)

    Returns:
        list: soil amplification factor, acceleration, T_B, T_C, T_D
    """
    # accelarations
    a_gR = {'1.1': 2.5, '1.2': 2.0, '1.3': 1.5, '1.4': 1.0, '1.5': 0.6, '1.6': 0.35, 
            '2.1': 2.5, '2.2': 2.0, '2.3': 1.7, '2.4': 1.1, '2.5': 0.8,
            '.1g': 0.980665, '.2g': 1.96133, '.3g':  2.941995, '.4g': 3.92266, '.5g': 4.903325, 
            '.6g': 5.88399, '.7g': 6.864655, '.8g': 7.84532, '.9g': 8.825985, '1.0g': 9.80665}
    
    index1 = ['i', 'ii', 'iii', 'iv']
    gama_f = {'CEN-1': [0.8, 1.0, 1.2, 1.4], 
            'CEN-2': [0.8, 1.0, 1.2, 1.4], 
            'PT-1': [0.65, 1.0, 1.45, 1.95], 
            'PT-2': [0.75, 1.0, 1.25, 1.5], 
            'PT-A': [0.85, 1.0, 1.15, 1.35]}
    coefs = pd.DataFrame(gama_f, index=index1)
    
    coef_imp = str.lower(coef_imp)
    a_g = a_gR[zone] * coefs.at[coef_imp, code]
    
    soil = str.upper(soil)
    index = ['A', 'B', 'C', 'D', 'E']
    code = str.upper(code)
    if code == 'CEN-1':
        data = {'S_max': [1.0 , 1.2, 1.15, 1.35, 1.4],
                'T_B': [0.15, 0.15, 0.2, 0.3, 0.15],
                'T_C': [0.4, 0.5, 0.6, 0.8, 0.5],
                'T_D': [2.0, 2.0, 2.0, 2.0, 2.0]
                }
    elif code == 'CEN-2':
        data = {'S_max': [1.0 , 1.35, 1.5, 1.88, 1.6],
                'T_B': [0.05, 0.05, 0.1, 0.1, 0.05],
                'T_C': [0.25, 0.25, 0.25, 0.3, 0.25],
                'T_D': [1.2, 1.2, 1.2, 1.2, 1.2]
                }
    elif code == 'PT-1':
        data = {'S_max': [1.0 , 1.35, 1.60, 2.0, 1.8],
                'T_B': [0.1, 0.1, 0.1, 0.1, 0.1],
                'T_C': [0.25, 0.25, 0.25, 0.3, 0.25],
                'T_D': [2.0, 2.0, 2.0, 2.0, 2.0]
                }
    elif code == 'PT-2' or code == 'PT-A':
        data = {'S_max': [1.0 , 1.35, 1.60, 2.0, 1.8],
                'T_B': [0.1, 0.1, 0.1, 0.1, 0.1],
                'T_C': [0.6, 0.6, 0.6, 0.8, 0.6],
                'T_D': [2.0, 2.0, 2.0, 2.0, 2.0]
                }
    values = pd.DataFrame(data, index=index)
    
    Smax = values.at[soil, 'S_max']

    if code == 'CEN-1' or code == 'CEN-2':
        S = Smax
    else: 
        if a_g <= 1.0:
            S = Smax 
        elif a_g >= 4.0:
            S = 1.0
        else:
            S = Smax-(Smax-1.0)*(a_g-1.0)/3.0

    TB = values.at[soil, 'T_B']
    TC = values.at[soil, 'T_C']
    TD = values.at[soil,'T_D']

    return S, a_g, TB, TC, TD


def get_spectrum(T: float, a_g: float, S: float, q: float, TB: float, TC: float, TD: float, beta: float=0.2) -> float:
    """Calculates the spectrum value for a given period

    Args:
        T (float): period (s)
        a_g (float): acceleration (m/s2)
        S (float): soil amplification factor
        q (float): behaviour factor
        TB (float): spectrum parameter
        TC (float): spectrum parameter
        TD (float): spectrum parameter
        beta (float, optional): the limiting value of the spectrum. Defaults to 0.2.

    Returns:
        float: the spectrum value
    """
    ag_S = a_g * S
    
    if T < TB:
        spec = ag_S * (2.0/3.0 + T / TB * (2.5 / q - 2.0/3.0))
    elif T < TC:
        spec = ag_S * 2.5 / q
    elif T < TD:
        spec = max(ag_S * 2.5 / q * (TC)/T, beta * ag_S)
    else:
        spec = max(ag_S * 2.5 / q * (TC*TD)/T**2, beta * ag_S)

    return spec


def spectrum_ec8(code: str, coef_imp: str, soil: str, zone:str, behaviour: float) -> pd.DataFrame:
    """Generate the spectrum DataFrame for the given parameters

    Args:
        code (str): the code of the spectrum (CEN-1, CEN-2, PT-1, PT-2, PT-A)
        coef_imp (str): importance coefficient (I, II, III, IV)
        soil (str): type of soil (A, B, C, D, E)
        zone (str): seismic zone 
        behaviour (float): behaviour factor

    Returns:
        pd.DataFrame: the spectrum DataFrame
    """
    txt = code + '_' + coef_imp + '_' + soil + '_' + zone + '_' + str(behaviour)

    S, a_g, TB, TC, TD = get_spectrum_parameters(code, coef_imp, soil, zone)

    periods = np.linspace(0.0, TB, 10, endpoint=False)
    periods = np.append(periods, np.linspace(TB, TC, 10, endpoint=False))
    periods = np.append(periods, np.linspace(TC, TD, 10, endpoint=False))
    periods = np.append(periods, np.linspace(TD, 10, 30))

    value = [get_spectrum(T, a_g, S, behaviour, TB, TC, TD, 0.2) for T in periods]
    data = {'period': periods,
            'value': value}
    
    spec = pd.DataFrame(data)
    spec.attrs['name'] = txt
    spec.attrs['S'] = S
    spec.attrs['a_g'] = a_g
    spec.attrs['q'] = str(behaviour)
    
    return spec


def spectrum_user(a_g: float, S: float, q: float, TB: float, TC: float, TD: float, beta: float=0.2) -> pd.DataFrame:
    """generate the spectrum DataFrame for the given parameters

    Args:
        a_g (float): acceleration (m/s2)
        S (float): soil amplification factor
        q (float): behaviour factor
        TB (float): spectrum parameter
        TC (float): spectrum parameter
        TD (float): spectrum parameter
        beta (float, optional): the limiting value of the spectrum. Defaults to 0.2.

    Returns:
        pd.DataFrame: _description_
    """
    txt = 'TB_' + str(TB) + '_TC_' + str(TC) + '_TD_' + str(TD) + '_b_' + str(beta)

    periods = np.linspace(0.0, TB, 10, endpoint=False)
    periods = np.append(periods, np.linspace(TB, TC, 10, endpoint=False))
    periods = np.append(periods, np.linspace(TC, TD, 10, endpoint=False))
    periods = np.append(periods, np.linspace(TD, 10, 30))

    value = [get_spectrum(T, a_g, S, q, TB, TC, TD, beta) for T in periods]
    data = {'period': periods,
            'value': value}

    spec = pd.DataFrame(data)
    spec.attrs['name'] = txt
    spec.attrs['S'] = S
    spec.attrs['a_g'] = round(a_g, 5)
    spec.attrs['q'] = q

    return spec


def write_spectrum_ec8(spectrum: pd.DataFrame, separator: str):
    """Generate a text file with the spectrum data

    Args:
        spectrum (pd.DataFrame): a pandas DataFrame with the spectrum data (columns: period, value)
        separator (str): a string with the separator to be used in the text file
    """
    # separator was defined as ' ' (space) for SAP2000 compatibility. any other can be used
    spectrum.to_csv('spectrum_' + spectrum.attrs['name'] + '.txt', index=False, sep=separator)


def draw_spectrum_ec8(spectrum: pd.DataFrame):
    """Draw the spectrum using matplotlib

    Args:
        spectrum (pd.DataFrame): a pandas DataFrame with the spectrum data (columns: period, value)
    """
    # plot the spectrum
    plt.plot(spectrum['period'].to_numpy(), spectrum['value'].to_numpy())
    s = spectrum.attrs['name'] + ':    S=' + str(spectrum.attrs['S']) + ' a_g=' + str(spectrum.attrs['a_g']) + ' q=' + str(spectrum.attrs['q'])
    plt.title(s)
    plt.xlabel('Period (s)')
    plt.ylabel('Spectrum value (m/s2)')


import math
import numpy as np
from typing import Tuple, Dict, List
import eurocodepy as ec


def calc_vrdmax(bw: float, d: float, fck: float, g_c: float, fyk: float, g_s: float, cott: float) -> float:
    """Calculates the design shear strength Vrd.max

    Args:
        bw (float): beam width
        d (float): beam depth
        fck (float): concrete compressive strength
        g_c (float): concrete partial safety coefficient
        fyk (float): steel strength
        g_s (float): steel partial safety coefficient

    Returns:
        float: Vrd.max
    """
    return bw * 0.9 * d * 0.6*(1.0-fck/250) * fck / g_c * 100.0 / (cott + 1.0/cott)


def calc_vrd(bw: float, d: float, fck: float, g_c: float, fyk: float, g_s: float, cott: float, asw_s: float, alpha: float) -> float:
    """Calculates the design shear strength Vrds and Vrd.max

    Args:
        bw (float): beam width
        d (float): beam depth
        fck (float): concrete compressive strength
        g_c (float): concrete partial safety coefficient
        fyk (float): steel strength
        g_s (float): steel partial safety coefficient
        cott (float): truss inclination (cot)
        asw_s (float): steel transverse area (Asw/s)
        alpha (float): coefficient

    Returns:
        float: (shear reinforcement max(Asw/s), Vrd.max)
    """
    z = 0.9 * d
    vrd_s = asw_s * z * fyk / g_s * cott * 1000.0
    niu = 0.6*(1.0-fck/250)
    vrd_max = bw * z * niu * fck / g_c * 100.0 / (cott + 1.0/cott)
    return max(vrd_s, vrd_max)


def calc_asws(bw: float, d: float, fck: float, g_c: float, fyk: float, g_s: float, cott: float, ved: float, alpha: float) -> Tuple[float, float]:
    """Calculates the design shear reinforcement

    Args:
        bw (float): beam width
        d (float): beam depth
        fck (float): concrete compressive strength
        g_c (float): concrete partial safety coefficient
        fyk (float): steel strength
        g_s (float): steel partial safety coefficient
        cott (float): truss inclination (cot)
        ved (float): design shear force
        alpha (float): coefficient

    Returns:
        Tuple[float, float]: (shear reinforcement (Asw/s), maximum shear force Vrd.max)
    """
    z = 0.9 * d
    niu = 0.6*(1.0-fck/250)
    vrd_max = bw * z * niu * fck / g_c * 1000.0 / (cott + 1.0/cott)

    asw_s = ved / z / fyk * g_s / cott / 1000.0 if vrd_max >= ved else math.nan
    return asw_s, vrd_max


def calc_vrdc(bw: float, d: float, fck: float, g_c: float, rho_l: float) -> Tuple[float, float, float]:
    """Shear strength without shear reinforcement

    Args:
        bw (float): beam width
        d (float): beam depth
        fck (float): concrete compressive strength
        g_c (float): concrete partial safety coefficient
        rho_l (float): longitudinal reinforcement ratio (As/bd)

    Returns:
        Tuple[float, float, float]: (vrd.min, vrd.c, vrd [min(vrd.mmin, vrd.c])
    """
    k = min(2.0, 1.0+math.sqrt(0.2/d))
    vrd_min = 35.0 * math.pow(k, 1.5) * math.sqrt(fck) * bw * d
    vrd_c = 180.0 / g_c * k * (100.0*rho_l*fck)**(1.0/3.0) * bw * d
    vrd = max (vrd_min, vrd_c)
    return vrd_min, vrd_c, vrd


def calc_asl(b: float, d: float, med: float, fcd: float=13.7, fyd: float=348.0, iprint: bool=False) -> Tuple[float, float, float]:
    """Calculates the reinforcement in a rectangular concrete beam.

    Args:
        b (float): bredth of the beam in m.
        d (float): depth of the reinforced in beam in m.
        med (float): bending moment in kNm.
        fcd (float, optional): concrete strength in MPa. Defaults to 20.0.
        fyd (float, optional): reinforcement strength in MPa. Defaults to 400.0.
        iprint (bool, optional): print results. Defaults to False.

    Returns:
        Tuple[float, float, float, float]: reinforcement area in cm2, strain in reinforcement, neutral axis depth, omega.
    """
    mmed = med
    bb = b
    dd = d
    miu = mmed/bb/d**2/fcd/1000.0
    try:
        omega = 1.0-math.sqrt(1-2*miu)
    except:
        omega = math.nan
    alpha = 1.25*omega
    ast = omega*b*d*fcd/fyd * 10000.0
    epss = (1.0-alpha)*3.5/alpha
    if iprint: 
        print("miu={:.3f} omega={:.3f} x/d={:.3f} eps-s={:.3f} As={:.2f} cm2".format(miu, round(omega,3), alpha, epss, ast))
    return ast, epss, alpha


def calc_mrd(b: float, d: float, ast: float, fcd: float=20.0, fyd: float=400.0, iprint: bool=False) -> Tuple[float, float, float]:
    """Calculates the bending moment in a rectangular concrete beam.

    Args:
        b (float): bredth of the beam in m.
        d (float): depth of the reinforced in beam in m.
        ast (float): reinforcement area in cm2.
        fcd (float, optional): concrete strength in MPa. Defaults to 20.0.
        fyd (float, optional): reinforcement strength in MPa. Defaults to 400.0.
        iprint (bool, optional): print results. Defaults to False.

    Returns:
        Tuple[float, float, float]: bending moment in kNm, strain in reinforcement, neutral axis depth.
    """
    omega = ast*fyd/b/d/fcd/10000.0
    miu = omega*(1-0.5*omega)
    alpha = 1.25*omega
    epss = (1.0-alpha)*3.5/alpha
    mrd = miu*b*d*d*fcd*1000
    if iprint: 
        print("med={:.1f} kNm; miu={:.3f}; omega={:.3f}".format(mrd, round(miu,3), round(omega,3)))
    return mrd, epss, alpha


def get_bend_params(conc:str='C20/25')->Tuple[float, float, float]:
    n = ec.ConcreteClasses[conc]['n']
    epsc2 = ec.ConcreteClasses[conc]['epsc2']
    epscu2 = ec.ConcreteClasses[conc]['epscu2']
    epsc12 = epsc2/epscu2
    chi1 = 1.0-epsc12/(n+1)
    chi2 = 1.0-((n+1)*(n+2)*0.5-epsc12**2)/((n+1)*(n+2)*chi1)
    chi = 0.5*chi1/chi2
    return chi1, chi2, chi


class RCBeam:
    def __init__(self, b: float, h: float, at: float = 0.05, ac: float = 0.05,
                conc: str="C25/30", gammac = None, reinf: str="B500B", gammas = None) -> None:
        """_summary_

        Args:
            b (float): bredth of the beam in m.
            h (float): height of the beam in m.
            conc (str, optional): concrete class. Defaults to "C25/30".
            reinf (str, optional): reinforcement lass. Defaults to "B500B".
            at (float, optional): tensile reinforcement mechanical cover. Defaults to 0.05 m.
            ac (float, optional): compressive reinforcement mechanical cover. Defaults to 0.05 m.
        """
        self.b = b
        self.h = h
        self.conc = conc
        self.reinf = reinf
        self.at = at
        self.ac = ac
        self.d = self.h - self.at
        self.fck = ec.ConcreteClasses[self.conc]['fck']
        self.fyk = ec.ReinforcementClasses[self.reinf]['fyk']
        self.gammac = gammac if gammac is not None else 1.5
        self.gammas = gammas if gammas is not None else 1.15
        self.fcd = self.fck/self.gammac
        self.fyd = self.fyk/self.gammas
        return
    
    def calcShear(self, med: float, ved: float, cott: float = 2.5, iprint: bool=False) -> Tuple[float, float, float]:
        aslt = 0.0
        aslc = 0.0
        alpha = 0.0
        epsst = 0.0
        epssc = 0.0
        asws = 0.0
        vrdmax = 0.0

        cotetas = np.arange(1.0, 2.5000000000000000000001, 0.1)
        vrdmaxs = calc_vrdmax(self.b, self.d, self.fck, self.gammac, self.fyk, self.gammas, cotetas)
        min_val = min(filter(lambda i: i > ved, vrdmaxs))

        asws = calc_asws(self.b, self.d, self.fck, self.gammac, self.fyk,self.gammas, ved)
        return aslt, aslc, alpha, epsst, epssc, asws, vrdmax
    
    def calcBending(self, med: float, delta: float = 1.0, iprint: bool=False) -> Tuple[float, float, float, float, float]:
        """Calculates the reinforcement area in a rectangular concrete beam.

        Args:
            med (float): design bending moment in kNm.
            delta (float, optional): redistribution ratio (0.7 to 1.0). Defaults to 1.0 (no redistribution).
            iprint (bool, optional): prints results to stdout. Defaults to False.

        Returns:
            Tuple[float, float, float, float, float]: reinforcement area in cm2, strain in reinforcement, neutral axis depth.
        """

        # Calculate med_max
        alpha_c = self.ac/self.d
        epscu2 = ec.ConcreteClasses[self.conc]['epscu2']

        alpha_lim = 0.45 if self.fck <= 50 else 0.35 # correct this values for fck > 50
        omega_max = alpha_lim/1.25
        miu_max = omega_max*(1-0.5*omega_max)
        med_max = miu_max*self.b*self.d*self.d*self.fck/self.gammac*1000.0
        
        # Check if miu is less than miu-max and calculate reinforcement accordingly
        if med <= med_max:
            # Calculate reinforcement for miu < miu-max
            aslt, epsst, alpha = calc_asl(self.b, self.d, med, self.fcd, self.fyd, iprint)
            aslc = None
            epssc = None
        else:
            # Calculate reinforcement for miu = miu-max
            epsyd = self.fyd/ec.ReinforcementClasses[self.reinf]['Es']*1000
            # epssc = (alpha_lim-alpha_c)/alpha_lim*epscu2
            # if epssc < epsyd:
            #     alpha_lim = epscu2/(epscu2-epsyd)*alpha_c
            #     omega_max = alpha_lim/1.25
            #     miu_max = omega_max*(1-0.5*omega_max)
            #     med_max = miu_max*self.b*self.d*self.d*self.fck/self.gammac*1000.0
            
            aslt, epsst, alpha = calc_asl(self.b, self.d, med_max, self.fcd, self.fyd, iprint)
            aslc = (med-med_max)/(self.d-self.ac)/self.fyd*10.0
            aslt += aslc
            epssc = (alpha-alpha_c)/alpha*epscu2

        
        return aslt, aslc, alpha, epsst, epssc

if __name__ == "__main__":
    # test RCBeam class
    print("\nTest RCBeam:\n")
    beam = RCBeam(0.3, 0.5, at=0.05, ac=0.05, conc="C70/85", reinf="A500NR")
    a = beam.calcShear(100.0, 100.0)
    asl, asc, a, epst, epsc = beam.calcBending(100.0)
    print (f"\n{asl=}, {asc=}, {a=}, {epst=}, {epsc=}\n")
    asl, asc, a, epst, epsc = beam.calcBending(500.0)
    print (f"\n{asl=}, {asc=}, {a=}, {epst=}, {epsc=}\n")
    asl, asc, a, epst, epsc = beam.calcBending(900.0)
    print (f"\n{asl=}, {asc=}, {a=}, {epst=}, {epsc=}\n")




import math
from typing import Tuple


def calc_vrd(bw: float, d: float, fck: float, g_c: float, fyk: float, g_s: float, cott: float, asw_s: float, alpha: float) -> float:
    """Calculates the design shear strength Vrds and Vrd.max

    Args:
        bw (float): beam width
        d (float): beam depth
        fck (float): concrete compressive strength
        g_c (float): concrete partial safety coefficient
        fyk (float): steel strength
        g_s (float): steel partial safety coefficient
        cott (float): truss inclination (cot)
        asw_s (float): steel transverse area (Asw/s)
        alpha (float): coefficient

    Returns:
        float: (shear reinforcement max(Asw/s), Vrd.max)
    """
    z = 0.9 * d
    vrd_s = asw_s * z * fyk / g_s * cott * 1000.0
    niu = 0.6*(1.0-fck/250)
    vrd_max = bw * z * niu * fck / g_c * 100.0 / (cott + 1.0/cott)
    return max(vrd_s, vrd_max)


def calc_vrdmax(bw: float, d: float, fck: float, g_c: float, fyk: float, g_s: float, cott: float) -> float:
    """Calculates the design shear strength Vrd.max

    Args:
        bw (float): beam width
        d (float): beam depth
        fck (float): concrete compressive strength
        g_c (float): concrete partial safety coefficient
        fyk (float): steel strength
        g_s (float): steel partial safety coefficient

    Returns:
        float: Vrd.max
    """
    return bw * 0.9 * d * 0.6*(1.0-fck/250) * fck / g_c * 100.0 / (cott + 1.0/cott)


def calc_asws(bw: float, d: float, fck: float, g_c: float, fyk: float, g_s: float, cott: float, ved: float, alpha: float) -> Tuple[float, float]:
    """Calculates the design shear reinforcement

    Args:
        bw (float): beam width
        d (float): beam depth
        fck (float): concrete compressive strength
        g_c (float): concrete partial safety coefficient
        fyk (float): steel strength
        g_s (float): steel partial safety coefficient
        cott (float): truss inclination (cot)
        ved (float): design shear force
        alpha (float): coefficient

    Returns:
        Tuple[float, float]: (shear reinforcement (Asw/s), maximum shear force Vrd.max)
    """
    z = 0.9 * d
    niu = 0.6*(1.0-fck/250)
    vrd_max = bw * z * niu * fck / g_c * 1000.0 / (cott + 1.0/cott)

    asw_s = ved / z / fyk * g_s / cott / 1000.0 if vrd_max >= ved else math.nan
    return asw_s, vrd_max


def calc_vrdc(bw: float, d: float, fck: float, g_c: float, rho_l: float) -> Tuple[float, float, float]:
    """Shear strength without shear reinforcement

    Args:
        bw (float): beam width
        d (float): beam depth
        fck (float): concrete compressive strength
        g_c (float): concrete partial safety coefficient
        rho_l (float): longitudinal reinforcement ratio (As/bd)

    Returns:
        Tuple[float, float, float]: (vrd.min, vrd.c, vrd [min(vrd.mmin, vrd.c])
    """
    k = min(2.0, 1.0+math.sqrt(0.2/d))
    vrd_min = 35.0 * math.pow(k, 1.5) * math.sqrt(fck) * bw * d
    vrd_c = 180.0 / g_c * k * (100.0*rho_l*fck)**(1.0/3.0) * bw * d
    vrd = max (vrd_min, vrd_c)
    return vrd_min, vrd_c, vrd